#include "cui.h"
#include "arg.h"
#include "readable.h"
#include "select.h"
#include "sock.h"
#include "focus.h"
#include "rect.h"
#include "esc.h"
#include "key.h"
#include "handler.h"
#include "panel.h"
#include "label.h"
#include "button.h"
#include "timer.h"
#include "list.h"
#include "menu.h"
#include "term.h"
#include "dialog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <termios.h>

#define KEY_FD	0
static int use_stdio = TRUE;
static fd_set sfds;
static int sfdn = 0;

static void
srv_accept(cui obj)
{
	int sfd, cfd;

	if(sfdn <= 0) return;
	for(sfd=0; sfd<FD_SETSIZE; sfd++){
		if(!FD_ISSET(sfd, &sfds)) continue;
		if(!cui_select_chk(sfd)) continue;
		if((cfd = cui_srv_accept(sfd)) == -1) ERR("srv_accept");
		cui_key_fd_add(cfd);
		cui_puts_temp_fd_enter(cfd);
		cui_esc_enter();
		cui_clear(obj);
		cui_draw(obj);
		cui_puts_temp_fd_exit();
	}
}

static void
conn(cui obj)
{
	char host[1024];
	int port, cfd;

	if(!cui_host_port_dialog(obj, 1, 1, host, &port)) return;
	if((cfd = cui_conn_host_port(host, port)) == -1){
		cui_msg_dialog(obj, "connect failed");
		return;
	}
	cui_key_fd_add(cfd);
	cui_puts_temp_fd_enter(cfd);
	cui_esc_enter();
	cui_clear(obj);
	cui_draw(obj);
	cui_puts_temp_fd_exit();
}

static void
disconn(void)
{
	int fd = cui_key_last_fd;
	if(fd == 0) return;
	cui_puts_temp_fd_enter(fd);
	cui_esc_exit();
	cui_puts_temp_fd_exit();
	cui_key_fd_del(fd);		
	close(fd);
}

int
cui_init(char **av)
{
	char **av2, *s;
	int port, sfd, fd;

	cui_dbg_open(av);
	cui_select_init();
	cui_key_init();

	av2 = av;
	while((s = cui_arg_str(av2, "-conn", NULL, &av2)) != NULL){
		if((fd = cui_conn(s)) == -1) continue;
		cui_key_fd_add(fd);
		use_stdio = cui_arg_exist(av, "-stdio", NULL);
	}
	FD_ZERO(&sfds);
	sfdn = 0;
	av2 = av;
	while((port = cui_arg_int(av2, "-srv", -1, &av2)) != -1){
		if((sfd = cui_srv_port(port)) == -1) ERR("cui_srv_port");
		FD_SET(sfd, &sfds);
		sfdn++;
		cui_select_add(sfd);
		use_stdio = cui_arg_exist(av, "-stdio", NULL);
	}
	if(use_stdio){
		cui_key_fd_add(KEY_FD);
		cui_key_enter();
	}
	cui_esc_enter();
	return 0;
}

void
cui_fini(int init_ret)
{
	int sfd;

	cui_esc_exit();
	cui_key_fd_close(); /* without KEY_FD (0) */
	for(sfd=0; sfd<FD_SETSIZE; sfd++){
		if(!FD_ISSET(sfd, &sfds)) continue;
		close(sfd);
		FD_CLR(sfd, &sfds);
	}
	if(use_stdio){
		cui_key_exit();
	}
	cui_dbg_close();
}

typedef struct cui_main_stack *cui_main_stack;

struct cui_main_stack{
	cui focus;
	int running;
	cui_main_stack prev;
};

static cui_main_stack main_stack = NULL;
cui_rect cui_clip = NULL;

cui
cui_focus_get(void)
{
	return main_stack ? main_stack->focus : NULL;
}

void
cui_focus_set(cui obj)
{
	if(main_stack) main_stack->focus = obj;
}

void
cui_focus_chk(void)
{
	cui f = cui_focus_get();
	if(f && (!cui_is_visible(f) || !cui_is_visible_rect(f))) cui_focus_set(NULL);
}

int
cui_running_get(void)
{
	return main_stack ? main_stack->running : FALSE;
}

void
cui_running_set(int v)
{
	if(main_stack) main_stack->running = v;
}

cui
cui_alloc(int size)
{
	cui obj;

	if((obj = malloc(size)) == NULL) ERR("No Mem");
	return obj;
}

void
cui_base_init(cui obj, cui parent, int x, int y, int w, int h)
{
	obj->parent = parent;
	obj->children = NULL;
	obj->next = NULL;
	obj->x = x;
	obj->y = y;
	obj->w = w;
	obj->h = h;

	if(parent) cui_base_child_add(parent, obj);
	obj->handler_list = NULL;
	obj->flags = 0;
}

void
cui_base_child_add(cui obj, cui child)
{
	child->next = obj->children;
	obj->children = child;
}

cui
cui_base_new(cui parent, int x, int y, int w, int h)
{
	cui obj = cui_alloc(sizeof(struct cui_base));
	cui_base_init(obj, parent, x, y, w, h);
	return obj;
}

void
cui_draw_str(cui obj, int x, int y, char *s, int attr)
{
	struct cui_rect t1, t2;
	cui_rect r1 = &t1, r2 = &t2;
	int n = strlen(s);
	char *s2 = NULL;

	if(!cui_is_visible(obj)) return;
	if(!cui_visible_rect(obj, r1)) return;
	cui_rect_init(r2, x, y, n, 1);
	if(!cui_rect_and(r1, r2, r2)) return; /* r1 and r2 --> r2 */
	if(cui_clip){
		*r1 = *cui_clip;
		r1->x = cui_lx(obj, r1->x);
		r1->y = cui_ly(obj, r1->y);
		if(!cui_rect_and(r1, r2, r2)) return;
	}

	s += r2->x - x;
	if(r2->x + r2->w < x + n){
		n = r2->w;
		if((s2 = malloc(n + 1)) == NULL) ERR("No Mem");
		memcpy(s2, s, n);
		s2[n] = '\0';
		s = s2;
	}
	x = r2->x;
	cui_esc_loc(cui_gx(obj) + x, cui_gy(obj) + y);
	if(attr & CUI_ATTR_ULINE) cui_esc_attr(CUI_ESC_ULINE);
	if(attr & CUI_ATTR_REVERSE) cui_esc_attr(CUI_ESC_REVERSE);
	cui_puts(s);
	if(attr != CUI_ATTR_NORMAL) cui_esc_attr(CUI_ESC_NORMAL);
	if(s2) free(s2);
}

void
cui_fill_rect(cui obj, int x, int y, int w, int h, char *s, int attr)
{
	int ix, iy;
	int n = strlen(s);
	char *buf;

	if(w <=0 || h <= 0) return;
	if((buf = malloc(w + 1)) == NULL) ERR("No Mem");
	for(ix=0; ix<w; ix+=n) buf[ix] = s[ix%n];
	buf[w] = '\0';
	for(iy=0; iy<h; iy++) cui_draw_str(obj, x, y+iy, buf, attr);
	free(buf);
}

void
cui_clear_rect(cui obj, int x, int y, int w, int h)
{
	cui_fill_rect(obj, x, y, w, h, " ", CUI_ATTR_NORMAL);
}

void
cui_clear(cui obj)
{
	cui_clear_rect(obj, 0, 0, obj->w, obj->h);
}

int
cui_gx(cui obj)
{
	return obj == NULL ? 0 : cui_gx(obj->parent) + obj->x;
}

int
cui_gy(cui obj)
{
	return obj == NULL ? 0 : cui_gy(obj->parent) + obj->y;
}

int
cui_glen(cui o1, cui o2)
{
	int dx, dy;

	dx = (cui_gx(o1) + o1->w / 2) - (cui_gx(o2) + o2->w / 2);
	dy = (cui_gy(o1) + o1->h / 2) - (cui_gy(o2) + o2->h / 2);
	dy *= 2;
	return dx * dx + dy * dy;
}

int
cui_lx(cui obj, int gx)
{
	return obj == NULL ? gx : cui_lx(obj->parent, gx) - obj->x;
}

int
cui_ly(cui obj, int gy)
{
	return obj == NULL ? gy : cui_ly(obj->parent, gy) - obj->y;
}

int
cui_x2(cui obj)
{
	return obj->x + obj->w;
}

int
cui_y2(cui obj)
{
	return obj->y + obj->h;
}

cui
cui_root(cui obj)
{
	return obj->parent == NULL ? obj : cui_root(obj->parent);
}

void
cui_draw(cui obj)
{
	int val;
	
	if(obj == NULL) return;
	if(!cui_is_visible(obj)) return;
	val = (obj == cui_focus_get()) ? CUI_DRAW_FOCUS : CUI_DRAW_NORMAL;
	cui_handler_call(obj, CUI_EVT_DRAW, val);
	//usleep(20*1000); // for debug
	cui_draw_chain(obj->children);
}

void
cui_draw_chain(cui obj)
{
	if(obj == NULL) return;
	cui_draw_chain(obj->next);
	cui_draw(obj);
}

void
cui_hide(cui obj)
{
	if(obj->flags & CUI_FLG_HIDE) return;
	cui_clear(obj);
	obj->flags |= CUI_FLG_HIDE;
}

void
cui_show(cui obj)
{
	if(!(obj->flags & CUI_FLG_HIDE)) return;
	obj->flags &= ~CUI_FLG_HIDE;
	cui_draw(obj);
}

void
cui_show_hide(cui obj, int v)
{
	if(v) cui_show(obj);
	else cui_hide(obj);
}

int
cui_is_visible(cui obj)
{
	if(obj == NULL) return TRUE; /* root */
	return !(obj->flags & CUI_FLG_HIDE) && cui_is_visible(obj->parent);
}

int
cui_visible_rect(cui obj, cui_rect r)
{
	struct cui_rect tmp;
	cui_rect t = &tmp;

	cui_rect_init(r, 0, 0, obj->w, obj->h);
	if(obj->parent == NULL) return !cui_rect_chk_empty(r); /* empty --> FALSE */
	if(!cui_visible_rect(obj->parent, t)) return FALSE;
	t->x -= obj->x;
	t->y -= obj->y;
	return cui_rect_and(r, t, r); /* r and t --> r */
}

int
cui_is_visible_rect(cui obj)
{
	struct cui_rect tmp;
	return cui_visible_rect(obj, &tmp);
}

int
cui_hide_enter(cui obj)
{
	int bak = cui_is_visible(obj);
	if(bak) cui_hide(obj);
	return bak;
}

void
cui_hide_exit(cui obj, int bak)
{
	if(bak) cui_show(obj);
}

void
cui_xywh_set(cui obj, int x, int y, int w, int h)
{
	int visi = cui_is_visible(obj);
	int rsz = (obj->w != w || obj->h != h);
	struct cui_rect r1, r2;
	cui_rects rs, ri;

	if(visi){
		if(obj->parent) cui_rect_init(&r1, cui_gx(obj), cui_gy(obj), obj->w, obj->h);
		else cui_clear(obj);
		obj->flags |= CUI_FLG_HIDE;
	}
	obj->x = x;
	obj->y = y;
	obj->w = w;
	obj->h = h;
	if(rsz) cui_handler_call(obj, CUI_EVT_RESIZE, TRUE);
	if(visi){
		obj->flags &= ~CUI_FLG_HIDE;
		if(obj->parent){
			cui_rect_init(&r2, cui_gx(obj), cui_gy(obj), obj->w, obj->h);
			rs = cui_rect_or(&r1, &r2);
			for(ri=rs; ri; ri=ri->next){
				cui_clip = (cui_rect)ri;
				cui_clear(obj->parent);
				cui_draw(obj->parent);
			}
			cui_clip = NULL;
			cui_rects_free(rs);
		}else{
			cui_draw(obj);
		}
	}
	if(obj == cui_focus_get() && !cui_is_visible_rect(obj)) cui_focus_set(NULL);
}

void
cui_x_set(cui obj, int x)
{
	cui_xywh_set(obj, x, obj->y, obj->w, obj->h);
}

void
cui_y_set(cui obj, int y)
{
	cui_xywh_set(obj, obj->x, y, obj->w, obj->h);
}

void
cui_xy_set(cui obj, int x, int y)
{
	cui_xywh_set(obj, x, y, obj->w, obj->h);
}

void
cui_w_set(cui obj, int w)
{
	cui_xywh_set(obj, obj->x, obj->y, w, obj->h);
}

void
cui_h_set(cui obj, int h)
{
	cui_xywh_set(obj, obj->x, obj->y, obj->w, h);
}

void
cui_wh_set(cui obj, int w, int h)
{
	cui_xywh_set(obj, obj->x, obj->y, w, h);
}

void
cui_wh_fit_chain(cui obj, cui child)
{
	if(child == NULL) return;
	if(obj->w < cui_x2(child)) obj->w = cui_x2(child);
	if(obj->h < cui_y2(child)) obj->h = cui_y2(child);
	cui_wh_fit_chain(obj, child->next);
}

void
cui_wh_fit(cui obj)
{
	int bak_w = obj->w, bak_h = obj->h;
	obj->w = obj->h = 0;
	cui_wh_fit_chain(obj, obj->children);
	if(obj->w != bak_w || obj->h != bak_h) cui_handler_call(obj, CUI_EVT_RESIZE, TRUE);
}

void
cui_wh_exp(cui obj)
{
	if(obj->parent)	cui_wh_set(obj, obj->parent->w - obj->x, obj->parent->h - obj->y);
}

int
cui_child_num(cui obj)
{
	return cui_index(obj->children) + 1;
}

int
cui_index(cui obj)
{
	if(obj == NULL) return -1;
	return cui_index(obj->next) + 1;
}

cui
cui_index_to_child(cui obj, int i)
{
	cui c = obj->children;
	int ci = cui_index(c);

	if(ci < 0 || ci < i) return NULL;
	while(ci-- > i) c = c->next;
	return c;
}

void
cui_quit(void)
{
	cui_running_set(FALSE);
}

static void
html_puts(char *s, void *prm)
{
	cui term = (cui)prm;
	cui_term_puts_buf(term, s);
}

static void
html_buf_flush(cui obj, int *attr, char *lp, int x, int y)
{
	cui_term p = (cui_term)obj;
	char *s = p->lbuf, c;

	if(*attr == -1) return;
	*lp = '\0';

	if(*attr & CUI_ATTR_ULINE) cui_dbg("<u>");
	if(*attr & CUI_ATTR_REVERSE) cui_dbg("<font color=\"#FFFFFF\" style=\"background-color:#000000\">");
	while((c = *s++) != '\0'){
		switch(c){
		case '&': cui_dbg("&amp;"); break;
		case '<': cui_dbg("&lt;"); break;
		case '>': cui_dbg("&gt;"); break;
		default: cui_dbg("%c", c); break;
		}
	}
	if(*attr & CUI_ATTR_REVERSE) cui_dbg("</font>");
	if(*attr & CUI_ATTR_ULINE) cui_dbg("</u>");
	*attr = -1;
}

static void
html_buf_out(cui obj)
{
	cui_term p = (cui_term)obj;
	char *lp = p->lbuf;
	int x, y, attr = -1, lx = 0;

	cui_dbg("<pre>\n");
	for(y=0; y<obj->h; y++){
		for(x=0; x<obj->w; x++){
			char *bp = cui_term_buf(obj, x, y);
			if(attr != -1 && bp[1] != attr) html_buf_flush(obj, &attr, lp, lx, y);
			if(attr == -1){
				lp = p->lbuf;
				*lp++ = bp[0] ? bp[0] : ' ';
				attr = bp[1];
				lx = x;
				continue;
			}
			/* bp[1] == attr */
			*lp++ = bp[0] ? bp[0] : ' ';
			continue;
			html_buf_flush(obj, &attr, lp, lx, y);
		}
		html_buf_flush(obj, &attr, lp, lx, y);
		cui_dbg("\n");
	}
	cui_dbg("</pre>\n");
}

static void
html(cui obj)
{
	cui term = cui_term_new(obj, 0, 0, cui_x2(obj), cui_y2(obj)+1);
	cui_hide(term);
	cui_draw(obj);

	cui_puts_add(html_puts, term);
	cui_draw(obj);

	cui_puts_del(html_puts);

	html_buf_out(term);

	cui_term_buf_free(term);
	cui_del_free(term);
}

static int
cmenu(cui obj, int key)
{
	cui mn;
	char *lst[] = { "Cancel", "^C", "Redraw", "Conn", "Disconn", "HTML", "Quit", NULL }, *s;

	if(key != CUI_KEY_CTRL('C')) return key;

	mn = cui_menu_popup_new(obj, 0, 1, -1, lst, 1);
	cui_hide(mn);
	cui_handler_call(mn, CUI_EVT_KEY, CUI_KEY_ENTER);
	s = cui_menu_popup_str_get(mn);
	cui_menu_popup_del_free(mn);

	if(strcmp(s, "^C") == 0) return 1;
	else if(strcmp(s, "Quit") == 0) cui_quit();
	else if(strcmp(s, "Redraw") == 0){
		cui_clear(obj);
		cui_draw(obj);
	}else if(strcmp(s, "HTML") == 0) html(cui_root(obj));
	else if(strcmp(s, "Conn") == 0) conn(obj);
	else if(strcmp(s, "Disconn") == 0) disconn();
	/* else Cancel */
	return 0;
}

void
cui_main(cui top_obj, cui init_focus)
{
	struct cui_main_stack stk;
	cui_main_stack sp = (cui_main_stack)&stk;

	int key, val;
	cui focus, obj;

	sp->focus = init_focus;
	sp->running = TRUE;
	sp->prev = main_stack;
	main_stack = sp;

	cui_clear(top_obj);
	cui_draw(top_obj);

	while(cui_running_get()){
		cui_timer_work();
		if(cui_select(100*1000) <= 0) continue;
		cui_readable_work();
		srv_accept(top_obj);
		if((key = cui_key_get2()) == 0) continue;
		if(cmenu(top_obj, key) == 0) continue;
		focus = cui_focus_get();
		if(focus){
			if(cui_handler_call(focus, CUI_EVT_KEY, key)) continue;
		}
		obj = cui_focus_move_judge(top_obj, key, NULL, &val);
		if(obj){
			cui_focus_set(obj);
			cui_draw(focus);
			cui_draw(obj);
		}
	}
	main_stack = sp->prev;
}

void
cui_del_from(cui obj, cui *from)
{
	if(*from == NULL) return;
	if(*from == obj){
		*from = obj->next;
		return;
	}
	cui_del_from(obj, &(*from)->next);
}

void
cui_del(cui obj)
{
	cui_del_from(obj, &obj->parent->children);
}

void
cui_free_chain(cui obj)
{
	if(obj == NULL) return;
	cui_free_chain(obj->next);
	cui_free(obj);
}

void
cui_free(cui obj)
{
	if(obj == NULL) return;
	cui_free_chain(obj->children);
	cui_list_del_all(&obj->handler_list);
	free(obj);
}

void
cui_del_free(cui obj)
{
	cui_del(obj);
	cui_free(obj);
}

typedef struct puts_data{
	puts_func f;
	void *prm;
} *puts_data;

static cui_list puts_list = NULL;

void
cui_puts_add(puts_func f, void *prm)
{
	puts_data p = cui_list_add(&puts_list, sizeof(struct puts_data));
	p->f = f;
	p->prm = prm;
}

void
cui_puts_del(puts_func f)
{
	cui_list lp = puts_list;
	for(; lp; lp=lp->next) if(*(puts_func *)lp->data == f) break;
	if(lp == NULL) return;
	cui_list_del(&puts_list, lp->data);
}

static void
temp_puts(char *s, void *prm)
{
	int fd = *(int *)prm, n = strlen(s);
	write(fd, s, n);
}

void
cui_puts_temp_fd_enter(int fd)
{
	static int tmp_fd;
	tmp_fd = fd;
	cui_puts_add(temp_puts, &tmp_fd);
}

void
cui_puts_temp_fd_exit(void)
{
	cui_puts_del(temp_puts);
}

void
cui_puts(char *s)
{
	cui_list lp = puts_list;
	if(puts_list == NULL){
		int fd, n = strlen(s);
		for(fd=0; fd<FD_SETSIZE; fd++){
			if(!cui_key_fd_chk(fd)) continue;
			write(fd, s, n);
		}
		return;
	}
	for(; lp; lp=lp->next){
		puts_data p = (puts_data)lp->data;
		(*p->f)(s, p->prm);
	}
}

/* EOF */
