#ifndef __CUI_H__
#define __CUI_H__

#define CUI_DRAW_NORMAL		0
#define CUI_DRAW_FOCUS		1
#define CUI_DRAW_ACTIVE		2

typedef struct cui_base *cui;

#include "dbg.h"

#define MSG(s) fprintf(stderr, "%s() %s L%d : %s\n", __func__, __FILE__, __LINE__, s)
#define ERR(s) do{ fprintf(stderr, "ERR "); MSG(s); exit(1); }while(0)

#define FALSE	0
#define TRUE	1

#define CUI_EVT_KEY		(1<<0)
#define CUI_EVT_DRAW		(1<<1)
#define CUI_EVT_BUTTON		(1<<2)
#define CUI_EVT_RESIZE		(1<<3)
#define CUI_EVT_TIMER		(1<<4)
#define CUI_EVT_READABLE	(1<<5)
#define CUI_EVT_DEAD		(1<<6)

#define CUI_FLG_CAN_FOCUS	(1<<0)
#define CUI_FLG_HIDE		(1<<1)

#define CUI_ATTR_NORMAL		0
#define CUI_ATTR_ULINE		(1<<0)
#define CUI_ATTR_REVERSE	(1<<1)

typedef int (*cui_handler)(cui obj, int evt, int val, void *prm);

#include "rect.h"
#include "list.h"

struct cui_base{
	cui parent, children, next;
	int x, y, w, h;
	cui_list handler_list;
	int flags;
};	

extern cui_rect cui_clip;

int cui_init(char **av);
void cui_fini(int init_ret);

cui cui_alloc(int size);
void cui_base_init(cui obj, cui parent, int x, int y, int w, int h);
void cui_base_child_add(cui obj, cui child);
cui cui_base_new(cui parent, int x, int y, int w, int h);

void cui_draw_str(cui obj, int x, int y, char *s, int attr);
void cui_fill_rect(cui obj, int x, int y, int w, int h, char *s, int attr);
void cui_clear_rect(cui obj, int x, int y, int w, int h);
void cui_clear(cui obj);

int cui_gx(cui obj);
int cui_gy(cui obj);
int cui_glen(cui o1, cui o2);
int cui_lx(cui obj, int gx);
int cui_ly(cui obj, int gy);
int cui_x2(cui obj);
int cui_y2(cui obj);
cui cui_root(cui obj);
void cui_draw(cui obj);
void cui_draw_chain(cui obj);
void cui_hide(cui obj);
void cui_show(cui obj);
void cui_show_hide(cui obj, int v);
int cui_is_visible(cui obj);
int cui_visible_rect(cui obj, cui_rect r);
int cui_is_visible_rect(cui obj);
int cui_hide_enter(cui obj);
void cui_hide_exit(cui obj, int bak);

void cui_xywh_set(cui obj, int x, int y, int w, int h);
void cui_x_set(cui obj, int x);
void cui_y_set(cui obj, int y);
void cui_xy_set(cui obj, int x, int y);
void cui_w_set(cui obj, int w);
void cui_h_set(cui obj, int h);
void cui_wh_set(cui obj, int w, int h);
void cui_wh_fit_chain(cui obj, cui child);
void cui_wh_fit(cui obj);
void cui_wh_exp(cui obj);

int cui_child_num(cui obj);
int cui_index(cui obj);
cui cui_index_to_child(cui obj, int i);

cui cui_focus_get(void);
void cui_focus_set(cui obj);
void cui_focus_chk(void);
int cui_running_get(void);
void cui_running_set(int v);

void cui_quit(void);
void cui_main(cui top_obj, cui init_focus);
void cui_del(cui obj);
void cui_free_chain(cui obj);
void cui_free(cui obj);
void cui_del_free(cui obj);

typedef void (*puts_func)(char *, void *prm);
void cui_puts_add(puts_func f, void *prm);
void cui_puts_del(puts_func f);
void cui_puts_temp_fd_enter(int fd);
void cui_puts_temp_fd_exit(void);
void cui_puts(char *s);

#endif
