#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "cui.h"
#include "arg.h"
#include "readable.h"
#include "sock.h"
#include "scpanel.h"
#include "tab.h"
#include "term.h"
#include "handler.h"
#include "button.h"
#include "dialog.h"
#include "menu.h"

struct conn_prm{
	cui view, menu_pd, tab;
};

static int
w_get(char **av)
{
	int v;
	char *cp;

	if((v = cui_arg_int(av, "-w", -1, NULL)) != -1) return v;
	if((cp = getenv("COLUMNS")) != NULL) return strtol(cp, NULL, 0);
	return 60;
}

static int
h_get(char **av)
{
	int v;
	char *cp;

	if((v = cui_arg_int(av, "-h", -1, NULL)) != -1) return v;
	if((cp = getenv("LINES")) != NULL) return strtol(cp, NULL, 0);
	return 20;
}

static void
close_show_hide(void *prm, int v)
{
	struct conn_prm *p = (struct conn_prm *)prm;
	cui menu = cui_menu_pulldown_menu_get(p->menu_pd);
	cui item = cui_menu_str_to_item(menu, "close");
	cui_show_hide(item, v);
}

static void
term_add(int cfd, void *prm)
{
	struct conn_prm *p = (struct conn_prm *)prm;
	int y = cui_y2(cui_tab_scline(p->tab));
	cui t = (cfd == -1) ? 
		cui_terminal_new(p->view, 0, 0, 0, 0):
		cui_termin_new(p->view, 0, 0, 0, 0, cfd);

	static int cnt = 0;
	char buf[16];

	cui_hide(t);
	cui_xywh_set(t, 0, y, p->view->w, p->view->h - y);

	sprintf(buf, "%s%d", cfd == -1 ? "sh-" : "", cnt++);
	cui_tab_add(p->tab, strdup(buf), t, FALSE);

	close_show_hide(prm, TRUE);
}

static int
tab_hdr(cui obj, int evt, int val, void *prm)
{
	int sfd, cfd;

	switch(evt){
	case CUI_EVT_READABLE:
		sfd = val;
		if((cfd = cui_srv_accept(sfd)) == -1) return FALSE;
		term_add(cfd, prm);
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

static int
view_hdr(cui obj, int evt, int val, void *prm)
{
	struct conn_prm *p = (struct conn_prm *)prm;
	cui tab = p->tab;
	cui termin, tab_btn, fc;

	switch(evt){
	case CUI_EVT_DEAD:
		termin = cui_index_to_child(obj, val);
		fc = cui_focus_get();
		if(fc == tab || fc == termin) cui_focus_set(NULL);
		cui_tab_del(tab, termin);
		cui_termin_del_free(termin);
		if(cui_tab_n_get(tab) <= 0) close_show_hide(prm, FALSE);
		break;
	case CUI_EVT_RESIZE:
		for(tab_btn=tab->children; tab_btn; tab_btn=tab_btn->next){
			termin = cui_tab_btn_sheet_get(tab_btn);
			cui_wh_set(termin, obj->w, obj->h - termin->y);
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

static int
conn_hdr(cui obj, int evt, int val, void *prm)
{
	struct conn_prm *p = (struct conn_prm *)prm;
	char host[1024];
	int port, cfd;

	switch(evt){
	case CUI_EVT_BUTTON:
		if(obj == p->menu_pd){
			char *s = cui_menu_pulldown_str_get(p->menu_pd);
			if(strcmp(s, "conn") == 0){
				if(!cui_host_port_dialog(p->view, 1, 1, host, &port)) break;
				if((cfd = cui_conn_host_port(host, port)) == -1){
					cui_msg_dialog(p->view, "connect failed");
					break;
				}
				term_add(cfd, prm);
			}else if(strcmp(s, "shell") == 0){
				term_add(-1, prm); /* shell */
			}else if(strcmp(s, "close") == 0){
				char buf[1024], *tab_str;
				cui t = cui_tab_get(p->tab);
				if(t== NULL) break;
				tab_str = cui_tab_str_get(p->tab);
				sprintf(buf, "close tab '%s' ?", tab_str);
				if(!cui_confirm_dialog(p->view, buf)) break;
				if(strncmp(tab_str, "sh-", 3) == 0) cui_terminal_kill(t);
				else cui_termin_kill(t);
			}
		}else return FALSE;
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

int
main(int ac, char **av)
{
	int init_ret = cui_init(av);
	cui bs = cui_scpanel_new(NULL, 0, 0, w_get(av), h_get(av), "cui_srv");
	cui view = ((cui_scpanel)bs)->view;
	char *lst[] = { "conn", "shell", "close", NULL };
	cui menu_pd = cui_menu_pulldown_new(view, 0, 0, "menu", lst);
	cui tab = cui_tab_new(view, cui_x2(menu_pd) + 1, 0, 0, NULL, NULL, FALSE);
	char **av2 = av;
	int port, sfd;
	struct conn_prm cprm = { view, menu_pd, tab };

	close_show_hide(&cprm, FALSE);
	cui_bind(tab, CUI_EVT_READABLE, tab_hdr, &cprm);
	while((port = cui_arg_int(av2, "-port", -1, &av2)) != -1){
		if((sfd = cui_srv_port(port)) == -1) continue;
		cui_readable_add(tab, sfd);
	}
	cui_bind(view, CUI_EVT_DEAD | CUI_EVT_RESIZE, view_hdr, &cprm);
	cui_bind(menu_pd, CUI_EVT_BUTTON, conn_hdr, &cprm);

	cui_main(bs, NULL);
	cui_free(bs);
	cui_fini(init_ret);

	// readable sfds close ...
	return 0;
}

/* EOF */
