#include "etext.h"
#include "menu.h"
#include "handler.h"
#include "key.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>

cui
cui_etext_new(cui parent, int x, int y, int w, char *init_s)
{
	cui obj = cui_alloc(sizeof(struct cui_etext));
	cui_etext_init(obj, parent, x, y, w, init_s);
	return obj;
}

void
cui_etext_init(cui obj, cui parent, int x, int y, int w, char *init_s)
{
	cui_etext p = (cui_etext)obj;

	cui_base_init(obj, parent, x, y, 0, 1);
	if(init_s == NULL) init_s = "";
	strcpy(p->buf, init_s);
	p->lb_top = cui_label_new(obj, 0, 0, "[");
	p->view = cui_base_new(obj, 1, 0, w-2, 1);
	p->lb_end = cui_label_new(obj, cui_x2(p->view), 0, "]");
	cui_wh_fit(obj);
	p->lbf = cui_lblfix_new(p->view, 0, 0, CUI_ETEXT_BSIZE, p->buf, CUI_ATTR_NORMAL);
	obj->flags |= CUI_FLG_CAN_FOCUS;

	p->mode = CUI_ETEXT_MODE_NORMAL;
	p->n = strlen(p->buf);
	p->cursor = 0;
	p->view_cursor = cui_base_new(p->lbf, 0, 0, 1, 1);
	p->lbf_shadow = cui_lblfix_new(p->view_cursor, 0, 0, CUI_ETEXT_BSIZE, p->buf, CUI_ATTR_REVERSE);
	cui_hide(p->view_cursor);

	p->hist_n = 0;
	cui_etext_hist_add(obj);

	cui_bind(obj, CUI_EVT_DRAW | CUI_EVT_KEY, cui_etext_hdr, NULL);
}

void
cui_etext_edit_in(cui obj, int key)
{
	cui_etext p = (cui_etext)obj;

	p->mode = CUI_ETEXT_MODE_EDIT;
	strcpy(p->bak, p->buf);
	if(key == CUI_KEY_BS) p->buf[0] = '\0';
	p->n = strlen(p->buf);

	cui_lblfix_attr_set(p->lbf, CUI_ATTR_NORMAL);
	cui_lblfix_str_set(p->lbf, p->buf);
	cui_lblfix_str_set(p->lbf_shadow, p->buf);

	p->cursor = (key == CUI_KEY_RIGHT) ? p->n : 0;
	cui_w_set(p->view_cursor, 1);
	cui_etext_cursor_update(obj);
	cui_show(p->view_cursor);
}

void
cui_etext_selall_in(cui obj)
{
	cui_etext p = (cui_etext)obj;
	p->cursor = 0;
	cui_etext_cursor_update(obj);
	cui_w_set(p->view_cursor, p->lbf->w);
	cui_show(p->view_cursor);
	p->mode = CUI_ETEXT_MODE_SELALL;
}

char *
cui_etext_hist_rbuf(cui obj, int i)
{
	cui_etext p = (cui_etext)obj;
	return p->hist[ i % CUI_ETEXT_HIST_LEN ];
}

void
cui_etext_hist_del(cui obj, char *str)
{
	cui_etext p = (cui_etext)obj;
	int i, s;

	s = p->hist_n <= CUI_ETEXT_HIST_LEN ? 0 : p->hist_n - CUI_ETEXT_HIST_LEN;
	for(i=p->hist_n-1; i>=s; i--){
		if(strcmp(cui_etext_hist_rbuf(obj, i), str) == 0) break;
	}
	if(i < s) return; /* not found */
	/* del */
	for(p->hist_n--; i < p->hist_n; i++){
		strcpy(cui_etext_hist_rbuf(obj, i), cui_etext_hist_rbuf(obj, i+1));
	}
}

void
cui_etext_hist_add(cui obj)
{
	cui_etext p = (cui_etext)obj;
	if(p->buf[0] == '\0') return;
	cui_etext_hist_del(obj, p->buf);
	strcpy(cui_etext_hist_rbuf(obj, p->hist_n++), p->buf);
}

int
cui_etext_hist_menu_hdr(cui pm, int evt, int val, void *prm)
{
	cui obj = (cui)prm;
	cui_etext p = (cui_etext)obj;

	strcpy(p->buf, cui_menu_popup_str_get(pm));
	cui_lblfix_str_set(p->lbf, p->buf);
	cui_lblfix_str_set(p->lbf_shadow, p->buf);
	p->n = strlen(p->buf);
	cui_etext_hist_add(obj);
	if(p->mode == CUI_ETEXT_MODE_SELALL){
		cui_hide(p->view_cursor);
		p->mode = CUI_ETEXT_MODE_NORMAL;
		cui_draw(obj);
	}
	else cui_etext_selall_in(obj);
	return TRUE;
}

void
cui_etext_hist_menu(cui obj)
{
	cui_etext p = (cui_etext)obj;
	char *lst[ CUI_ETEXT_HIST_LEN + 1 ];
	int i, s, j;
	cui pm;

	if(p->hist_n <= 0) return;

	s = p->hist_n <= CUI_ETEXT_HIST_LEN ? 0 : p->hist_n - CUI_ETEXT_HIST_LEN;
	j = 0;
	for(i=p->hist_n-1; i>=s; i--) lst[j++] = cui_etext_hist_rbuf(obj, i);
	lst[j] = NULL;

	pm = cui_menu_popup_new(obj->parent, obj->x+1, obj->y+1, 0, lst, 0);
	cui_bind(pm, CUI_EVT_BUTTON, cui_etext_hist_menu_hdr, p);
	cui_hide(pm);
	cui_handler_call(pm, CUI_EVT_KEY, CUI_KEY_ENTER);
	cui_menu_popup_del_free(pm);
}

int
cui_etext_digit_menu_hdr(cui pm, int evt, int val, void *prm)
{
	cui obj = (cui)prm;
	cui_etext p = (cui_etext)obj;

	p->buf[p->cursor] = '0' + cui_menu_int_get(pm);
	cui_lblfix_str_set(p->lbf, p->buf);
	cui_lblfix_str_set(p->lbf_shadow, p->buf);
	return TRUE;
}

void
cui_etext_digit_menu(cui obj)
{
	cui_etext p = (cui_etext)obj;
	cui pm;

	if(p->cursor >= p->n || !isdigit(p->buf[p->cursor])) return;
	pm = cui_menu_int_new(p->view_cursor, 0, 0, 0, p->buf[p->cursor] - '0', 0, 9);
	cui_bind(pm, CUI_EVT_BUTTON, cui_etext_digit_menu_hdr, p);
	cui_hide(pm);
	cui_handler_call(pm, CUI_EVT_KEY, CUI_KEY_ENTER);
	cui_menu_int_del_free(pm);	
}

void
cui_etext_edit_out(cui obj)
{
	cui_etext p = (cui_etext)obj;

	cui_hide(p->view_cursor);
	cui_lblfix_str_set(p->lbf, p->buf);
	cui_lblfix_attr_set(p->lbf, CUI_ATTR_ULINE);
	cui_x_set(p->lbf, 0);
	p->mode = CUI_ETEXT_MODE_NORMAL;
	if(strcmp(p->buf, p->bak) != 0){
		cui_etext_hist_add(obj);
		cui_handler_call(obj, CUI_EVT_BUTTON, 0);
	}
}

void
cui_etext_cursor_update(cui obj)
{
	cui_etext p = (cui_etext)obj;

	cui_hide(p->view_cursor);
	cui_draw(p->view_cursor->parent);
	cui_x_set(p->view_cursor, p->cursor);
	cui_x_set(p->lbf_shadow, -p->cursor);
	while(cui_lx(p->view, cui_gx(p->view_cursor)) < 0){
		cui_x_set(p->lbf, p->lbf->x + 1);
	}
	while(cui_lx(p->view, cui_gx(p->view_cursor)) >= p->view->w){
		cui_x_set(p->lbf, p->lbf->x - 1);
	}
	cui_show(p->view_cursor);
}

static void
mode_edit_normal_key_in(cui obj, int val)
{
	cui_etext p = (cui_etext)obj;
	int i;

	if(p->n >= CUI_ETEXT_BSIZE - 1) return;
	for(i=p->n; i>=p->cursor; i--) p->buf[i+1] = p->buf[i];
	p->buf[p->cursor] = val;
	p->n++;
	cui_lblfix_str_set(p->lbf, p->buf);
	cui_lblfix_str_set(p->lbf_shadow, p->buf);
	p->cursor++;
	cui_etext_cursor_update(obj);
}

int
cui_etext_hdr(cui obj, int evt, int val, void *prm)
{
	cui_etext p = (cui_etext)obj;
	int attr, i;

	switch(evt){
	case CUI_EVT_DRAW:
		attr = (val == CUI_DRAW_FOCUS) ? CUI_ATTR_ULINE : CUI_ATTR_NORMAL;
		cui_label_attr_set(p->lb_top, attr);
		cui_lblfix_attr_set(p->lbf, attr);
		cui_label_attr_set(p->lb_end, attr);
		return TRUE;
	case CUI_EVT_KEY:
		switch(p->mode){
		case CUI_ETEXT_MODE_NORMAL:
			if(val == CUI_KEY_ENTER){
				if(p->buf[0] == '\0') cui_etext_edit_in(obj, val);
				else cui_etext_selall_in(obj);
				return TRUE;
			}
			return FALSE;
		case CUI_ETEXT_MODE_SELALL:
			switch(val){
			case CUI_KEY_ESC:
				cui_hide(p->view_cursor);
				p->mode = CUI_ETEXT_MODE_NORMAL;
				cui_draw(obj);
				break;
			case CUI_KEY_ENTER:
			case CUI_KEY_LEFT:
			case CUI_KEY_RIGHT:
			case CUI_KEY_BS:
				cui_etext_edit_in(obj, val);
				break;
			case CUI_KEY_DOWN:
				cui_etext_hist_menu(obj);
				break;
			default:
				cui_etext_edit_in(obj, CUI_KEY_BS);
				mode_edit_normal_key_in(obj, val);
				break;				
			}
			return TRUE;
		case CUI_ETEXT_MODE_EDIT:
			switch(val){
			case CUI_KEY_ESC:
				strcpy(p->buf, p->bak);
			case CUI_KEY_ENTER:
				cui_etext_edit_out(obj);
				break;
			case CUI_KEY_RIGHT:
				if(p->cursor >= p->n) break;
				p->cursor++;
				cui_etext_cursor_update(obj);
				break;
			case CUI_KEY_LEFT:
				if(p->cursor <= 0) break;
				p->cursor--;
				cui_etext_cursor_update(obj);
				break;
			case CUI_KEY_UP:
				cui_etext_digit_menu(obj);
				break;
			case CUI_KEY_DOWN:
				cui_etext_hist_menu(obj);
				break;
			case CUI_KEY_BS:
				if(p->cursor <= 0) break;
				for(i=p->cursor; i<=p->n; i++) p->buf[i-1] = p->buf[i];
				p->n--;
				cui_lblfix_str_set(p->lbf, p->buf);
				cui_lblfix_str_set(p->lbf_shadow, p->buf);
				p->cursor--;
				cui_etext_cursor_update(obj);
				break;
			default:
				mode_edit_normal_key_in(obj, val);
				break;
			}
			return TRUE;
		}
	}
	return FALSE;
}

char *
cui_etext_str_get(cui obj)
{
	cui_etext p = (cui_etext)obj;
	return p->buf;
}

void
cui_etext_str_set(cui obj, char *s)
{
	cui_etext p = (cui_etext)obj;
	while(p->mode != CUI_ETEXT_MODE_NORMAL) cui_handler_call(obj, CUI_EVT_KEY, CUI_KEY_ESC);
	strcpy(p->buf, s);
	cui_lblfix_str_set(p->lbf, p->buf);
}

/* EOF */
