#include "key.h"
#include "select.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <termios.h>

#define KEY_FD	0

static struct termios bak;
fd_set cui_key_fds;

void
cui_key_init(void)
{
	FD_ZERO(&cui_key_fds);
}

void
cui_key_enter(void)
{
	struct termios raw;

	tcgetattr(KEY_FD, &bak);
	cfmakeraw(&raw);
	raw.c_lflag &= ~FLUSHO;
	tcsetattr(KEY_FD, TCSANOW, &raw);
}

void
cui_key_exit(void)
{
	tcsetattr(KEY_FD, TCSANOW, &bak);
}

void
cui_key_fd_add(int fd)
{
	FD_SET(fd, &cui_key_fds);
	cui_select_add(fd);
}

void
cui_key_fd_del(int fd)
{
	FD_CLR(fd, &cui_key_fds);
	cui_select_del(fd);
}

int
cui_key_fd_chk(int fd)
{
	return FD_ISSET(fd, &cui_key_fds);
}

int
cui_key_get(void)
{
	unsigned char uc;

	if(!cui_select_readable_chk(KEY_FD, 100*1000)) return 0;
	if(read(KEY_FD, &uc, 1) != 1) return 0;
	return uc;
}

void
cui_key_fd_close(void) /* without KEY_FD (0) */
{
	int fd;
	for(fd=1; fd<FD_SETSIZE; fd++){
		if(!cui_key_fd_chk(fd)) continue;
		cui_key_fd_del(fd);
		close(fd);
	}
}

int cui_key_last_fd = -1;

static int
key_get(int fd)
{
	unsigned char uc;

	if(!cui_select_readable_chk(fd, 100*1000)) return 0;
	if(read(fd, &uc, 1) != 1) return 0;
	cui_key_last_fd = fd;
	return uc;
}

int
cui_key_get2(void)
{
	int fd;
	int k, k2, k3;

	for(fd=0; fd<FD_SETSIZE; fd++){
		if(!cui_key_fd_chk(fd)) continue;
		if(cui_select_readable_chk(fd, 0)) break;
	}
	if(fd >= FD_SETSIZE) return 0;

	k = key_get(fd);
	if(k == 0x7f) return CUI_KEY_BS;
	if(k == CUI_KEY_ESC){
		k2 = key_get(fd);
		if(k2 == '['){
			k3 = key_get(fd);
			switch(k3){
			case 'A': return CUI_KEY_UP;
			case 'B': return CUI_KEY_DOWN;
			case 'C': return CUI_KEY_RIGHT;
			case 'D': return CUI_KEY_LEFT;
			}
		}
	}
	return k;
}

/* EOF */
