#include "menu.h"
#include "handler.h"
#include "key.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

cui
cui_menu_btn_new(cui parent, int x, int y, char *s)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_btn));
	cui_menu_btn_init(obj, parent, x, y, s);
	return obj;
}

void
cui_menu_btn_init(cui obj, cui parent, int x, int y, char *s)
{
	cui_button btn = (cui_button)obj;

	cui_button_init(obj, parent, x, y, s);
	cui_bind(obj, CUI_EVT_DRAW, cui_menu_btn_hdr, NULL);
	cui_hide(btn->lb1);
	cui_hide(obj);
	cui_label_str_set(btn->lb3, "|");
	btn->lb2->x -= 1;
	btn->lb3->x -= 1;
	obj->w = 0;
	cui_wh_fit(obj);
	cui_label_attr_set(btn->lb2, CUI_ATTR_ULINE);
	cui_show(obj);
}

int
cui_menu_btn_hdr(cui obj, int evt, int val, void *prm)
{
	cui_button btn = (cui_button)obj;
	int attr = CUI_ATTR_ULINE;
	if(val == CUI_DRAW_FOCUS) attr |= CUI_ATTR_REVERSE;
	cui_label_attr_set(btn->lb2, attr);
	return TRUE;
}

char *
cui_menu_btn_str_get(cui obj)
{
	return cui_button_str_get(obj);
}

void
cui_menu_btn_str_set(cui obj, char *s)
{
	cui_button_str_set(obj, s);
}

cui
cui_menu_item_new(cui parent, int x, int y, char *s)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_item));
	cui_menu_item_init(obj, parent, x, y, s);
	return obj;
}

void
cui_menu_item_init(cui obj, cui parent, int x, int y, char *s)
{
	cui_menu_item p = (cui_menu_item)obj;

	cui_base_init(obj, parent, x, y, 0, 1);
	cui_hide(obj);
	p->lbf = cui_lblfix_new(obj, 0, 0, -1, s, CUI_ATTR_ULINE);
	p->lb_end = cui_label_new(obj, cui_x2(p->lbf), 0, "|");
	cui_bind(obj, CUI_EVT_DRAW | CUI_EVT_KEY | CUI_EVT_RESIZE, cui_menu_item_hdr, NULL);
	cui_wh_fit(obj);
	cui_show(obj);
	obj->flags |= CUI_FLG_CAN_FOCUS;
}

int
cui_menu_item_hdr(cui obj, int evt, int val, void *prm)
{
	cui_menu_item p = (cui_menu_item)obj;
	int attr, dir;
	cui obj2;

	switch(evt){
	case CUI_EVT_BUTTON:
		val = val ? cui_index(obj) : -1;
		cui_handler_call(obj->parent, CUI_EVT_BUTTON, val);
		return TRUE;
	case CUI_EVT_KEY:
		switch(val){
		case CUI_KEY_ENTER:
		case CUI_KEY_ESC:
			cui_handler_call(obj, CUI_EVT_BUTTON, val == CUI_KEY_ENTER);
			return TRUE;
		case CUI_KEY_DOWN:
		case CUI_KEY_UP:
			dir = val == CUI_KEY_DOWN ? 1 : -1;
			obj2 = cui_index_to_child(obj->parent, cui_index(obj) + dir);
			if(obj2 && !cui_is_visible_rect(obj2)){
				cui_y_set(obj->parent, obj->parent->y - dir);
			}
			break;
		}
		return FALSE;
	case CUI_EVT_DRAW:
		attr = CUI_ATTR_ULINE;
		if(val == CUI_DRAW_FOCUS) attr |= CUI_ATTR_REVERSE;
		cui_lblfix_attr_set(p->lbf, attr);
		return TRUE;
	case CUI_EVT_RESIZE:
		cui_w_set(p->lbf, obj->w - 1);
		cui_x_set(p->lb_end, cui_x2(p->lbf));
		return TRUE;
	}
	return FALSE;
}

char *
cui_menu_item_str_get(cui obj)
{
	cui_menu_item p = (cui_menu_item)obj;
	return cui_lblfix_str_get(p->lbf);
}

void
cui_menu_item_str_set(cui obj, char *s)
{
	int bak = cui_hide_enter(obj);
	cui_menu_item p = (cui_menu_item)obj;
	cui_lblfix_str_set(p->lbf, s);
	cui_hide_exit(obj, bak);
}

cui
cui_menu_new(cui parent, int x, int y, char **lst)
{
	cui obj = cui_base_new(parent, x, y, 0, 0);
	char *s;
	cui item;
	
	cui_hide(obj);
	y = 0;
	while((s = *lst++) != NULL) cui_menu_item_new(obj, 0, y++, s);
	cui_wh_fit(obj);
	for(item=obj->children; item; item=item->next){	
		cui_w_set(item, obj->w);
		cui_bind(item, CUI_EVT_BUTTON, cui_menu_item_hdr, NULL);
	}
	return obj;
}

cui
cui_menu_str_to_item(cui menu, char *s)
{
	cui item;

	for(item=menu->children; item; item=item->next){
		if(strcmp(cui_menu_item_str_get(item), s) == 0) return item;
	}
	return NULL;
}

int
cui_menu_item_num(cui obj)
{
	return cui_child_num(obj);
}

cui
cui_menu_item_get(cui obj, int i)
{
	return cui_index_to_child(obj, i);
}

cui
cui_menu_popup_new(cui parent, int x, int y, int w, char **lst, int val)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_popup));
	cui_menu_popup_init(obj, parent, x, y, w, lst, val);
	return obj;
}

void
cui_menu_popup_init(cui obj, cui parent, int x, int y, int w, char **lst, int val)
{
	cui_menu_popup p = (cui_menu_popup)obj;

	cui_menu_item_init(obj, parent, x, y, lst[val]);
	if(w >= 0) cui_w_set(obj, w);
	p->menu = cui_menu_new(cui_root(obj), 0, 0, lst);
	p->val = val;
	p->vp = NULL;
	p->call_btn_hdr_flag = TRUE; /* ! */

	cui_unbind(obj, CUI_EVT_KEY | CUI_EVT_BUTTON, cui_menu_item_hdr);
	cui_bind(obj, CUI_EVT_KEY, cui_menu_popup_hdr, NULL);
	cui_bind(p->menu, CUI_EVT_BUTTON, cui_menu_popup_hdr, obj);
}

int
cui_menu_popup_hdr(cui obj, int evt, int val, void *prm)
{
	if(evt == CUI_EVT_KEY && val == CUI_KEY_ENTER){ /* menu_popup */
		cui_menu_popup p = (cui_menu_popup)obj;
		cui menu = p->menu;
		int bak = p->val;
		menu->x = cui_lx(cui_root(obj), cui_gx(obj));
		menu->y = cui_ly(cui_root(obj), cui_gy(obj)) - p->val;
		menu->flags &= ~CUI_FLG_HIDE;
		cui_main(menu, cui_index_to_child(menu, p->val));
		cui_hide(menu);
		if(p->val < 0) p->val = bak;
		else{
			char *s = cui_menu_item_str_get(cui_index_to_child(menu, p->val));
			cui_menu_item_str_set(obj, s);
			if(p->call_btn_hdr_flag) cui_handler_call(obj, CUI_EVT_BUTTON, p->val);
		}
		cui_draw(menu->parent);
		return TRUE;
	}else if(evt == CUI_EVT_BUTTON){ /* menu */
		cui_menu_popup p = (cui_menu_popup)prm;
		p->val = val;
		if(p->vp) *p->vp = p->val;
		cui_quit();
		return TRUE;
	}
	return FALSE;
}

char *
cui_menu_popup_str_get(cui obj)
{
	return cui_menu_item_str_get(obj);
}

int
cui_menu_popup_get(cui obj)
{
	cui_menu_popup p = (cui_menu_popup)obj;
	return p->val;
}

void
cui_menu_popup_set(cui obj, int idx)
{
	cui_menu_popup p = (cui_menu_popup)obj;
	char *s;
	p->val = idx;
	if(p->vp) *p->vp = p->val;
	s = cui_menu_item_str_get(cui_index_to_child(p->menu, p->val));
	cui_menu_item_str_set(obj, s);
}

void
cui_menu_popup_del_free(cui obj)
{
	cui_menu_popup p = (cui_menu_popup)obj;
	cui_del_free(p->menu);
	cui_del_free(obj);
}

void
cui_menu_popup_vp_set(cui obj, int *vp)
{
	cui_menu_popup p = (cui_menu_popup)obj;
	p->vp = vp;
	if(p->vp) cui_menu_popup_set(obj, *p->vp);
}

cui
cui_menu_int_new(cui parent, int x, int y, int w, int init_v, int min_v, int max_v)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_int));
	cui_menu_int_init(obj, parent, x, y, w, init_v, min_v, max_v);
	return obj;
}

static int
menu_int_alloc_size(int min_v, int max_v)
{
	char buf[32];
	int n = max_v - min_v + 1;
	int sum = sizeof(char *) * (n + 1);
	int v, i;

	for(i=0; i<n; i++){
		v = max_v - i;
		sprintf(buf, "%d", v);
		sum += strlen(buf) + 1;
	}
	return sum;
}

static void
menu_int_lst_setup(cui obj)
{
	cui_menu_int p = (cui_menu_int)obj;
	int n = p->max_v - p->min_v + 1;
	char *cp = (char *)&p->lst[n + 1];
	int i, v;

	for(i=0; i<n; i++){
		p->lst[i] = cp;
		v = p->max_v - i;
		sprintf(cp, "%d", v);
		cp += strlen(cp) + 1;
	}
	p->lst[i] = NULL;
}

void
cui_menu_int_init(cui obj, cui parent, int x, int y, int w, int init_v, int min_v, int max_v)
{
	cui_menu_int p = (cui_menu_int)obj;

	if((p->alloc = malloc(menu_int_alloc_size(min_v, max_v))) == NULL) ERR("No Mem");
	p->lst = (char **)p->alloc;
	p->min_v = min_v;
	p->max_v = max_v;	
	menu_int_lst_setup(obj);
	p->v = init_v;
	p->vp = NULL;

	cui_menu_popup_init(obj, parent, x, y, w, p->lst, max_v - init_v);
	p->popup.call_btn_hdr_flag = FALSE; /* ! */

	cui_unbind(obj, CUI_EVT_KEY, cui_menu_popup_hdr);
	cui_bind(obj, CUI_EVT_KEY, cui_menu_int_hdr, NULL);
}

void
cui_menu_int_lst_free(cui obj)
{
	cui_menu_int p = (cui_menu_int)obj;
	if(p->alloc){
		free(p->alloc);
		p->alloc = NULL;
	}
}

static void
menu_int_set(cui obj , int v)
{
	cui_menu_int p = (cui_menu_int)obj;
	if(v < p->min_v) v = p->min_v;
	if(v > p->max_v) v = p->max_v;
	p->v = v;
	if(p->vp) *p->vp = p->v;
}

int
cui_menu_int_hdr(cui obj, int evt, int val, void *prm)
{
	/* CUI_EVT_KEY */

	cui_menu_int p = (cui_menu_int)obj;
	int bak = p->popup.val;
	if(cui_menu_popup_hdr(obj, evt, val, prm)){
		if(p->popup.val != bak){
			menu_int_set(obj, p->max_v - p->popup.val);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}
		return TRUE;
	}
	return FALSE;
}

int
cui_menu_int_get(cui obj)
{
	cui_menu_int p = (cui_menu_int)obj;
	return p->v;	
}

void
cui_menu_int_set(cui obj, int v)
{
	cui_menu_int p = (cui_menu_int)obj;
	menu_int_set(obj, v);
	cui_menu_popup_set(obj, p->max_v - p->v);
}

void
cui_menu_int_vp_set(cui obj, int *vp)
{
	cui_menu_int p = (cui_menu_int)obj;
	p->vp = vp;
	if(p->vp) cui_menu_int_set(obj, *p->vp);
}

void
cui_menu_int_min_max_set(cui obj, int min_v, int max_v)
{
	cui_menu_int p = (cui_menu_int)obj;

	cui_menu_int_lst_free(obj);

	if((p->alloc = malloc(menu_int_alloc_size(min_v, max_v))) == NULL) ERR("No Mem");
	p->lst = (char **)p->alloc;
	
	p->min_v = min_v;
	p->max_v = max_v;	
	menu_int_lst_setup(obj);
	cui_menu_int_set(obj, p->v);
}

void
cui_menu_int_del_free(cui obj)
{
	cui_menu_int_lst_free(obj);
	cui_menu_popup_del_free(obj);
}

cui
cui_menu_pulldown_new(cui parent, int x, int y, char *s, char **lst)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_pulldown));
	cui_menu_pulldown_init(obj, parent, x, y, s, lst);
	return obj;
}

void
cui_menu_pulldown_init(cui obj, cui parent, int x, int y, char *s, char **lst)
{
	cui_menu_pulldown p = (cui_menu_pulldown)obj;
	cui_base_init(obj, parent, x, y, 0, 0);
	p->btn = cui_menu_btn_new(obj, 0, 0, s);
	cui_wh_fit(obj);
	p->menu = cui_menu_new(cui_root(obj), 0, 0, lst);
	p->v = -1;
	p->vp = NULL;

	cui_bind(p->btn, CUI_EVT_BUTTON, cui_menu_pulldown_hdr, obj);
	cui_bind(p->menu, CUI_EVT_BUTTON, cui_menu_pulldown_hdr, obj);
}

int
cui_menu_pulldown_hdr(cui ev_obj, int evt, int val, void *prm)
{
	cui obj = (cui)prm;
	cui_menu_pulldown p = (cui_menu_pulldown)obj;

	switch(evt){
	case CUI_EVT_BUTTON:
		if(ev_obj == p->btn){
			int bak = p->v;
			p->menu->x = cui_lx(cui_root(obj), cui_gx(obj));
			p->menu->y = cui_ly(cui_root(obj), cui_gy(obj)) + 1;
			p->menu->flags &= ~CUI_FLG_HIDE;
			cui_main(p->menu, cui_index_to_child(p->menu, 0));
			cui_hide(p->menu);
			if(p->v < 0) p->v = bak;
			else cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}else if(ev_obj == p->menu){
			p->v = val;
			if(p->vp) *p->vp = p->v;
			cui_quit();
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

int
cui_menu_pulldown_get(cui obj)
{
	cui_menu_pulldown p = (cui_menu_pulldown)obj;
	return p->v;
}

void
cui_menu_pulldown_vp_set(cui obj, int *vp)
{
	cui_menu_pulldown p = (cui_menu_pulldown)obj;
	p->vp = vp;
}

cui
cui_menu_pulldown_menu_get(cui obj)
{
	cui_menu_pulldown p = (cui_menu_pulldown)obj;
	return p->menu;
}

char *
cui_menu_pulldown_str_get(cui obj)
{
	cui_menu_pulldown p = (cui_menu_pulldown)obj;
	cui item;

	if(p->v < 0) return "";
	if((item = cui_menu_item_get(p->menu, p->v)) == NULL) return "";
	return cui_menu_item_str_get(item);
}

void
cui_menu_pulldown_del_free(cui obj)
{
	cui_menu_pulldown p = (cui_menu_pulldown)obj;
	cui_del_free(p->menu);
	cui_del_free(obj);
}

/* EOF */
