#include "num.h"
#include "etext.h"
#include "fillbtn.h"
#include "scbar.h"
#include "handler.h"
#include "key.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

cui
cui_num_new(cui parent, int x, int y, int w, int init_v, int min_v, int max_v, int bar_h)
{
	cui obj = cui_alloc(sizeof(struct cui_num));
	cui_num_init(obj, parent, x, y, w, init_v, min_v, max_v, bar_h);
	return obj;
}

static int
bar_key_hdr(cui obj, int evt, int val, void *prm)
{
	/* CUI_EVT_KEY */

	cui num = (cui)prm;
	cui_num p = (cui_num)num;
	cui_scbar scb = (cui_scbar)p->bar;

	if(scb->thumb_drag) return FALSE;
	switch(val){
	case CUI_KEY_ESC:
	case CUI_KEY_LEFT:
	case CUI_KEY_RIGHT:
		cui_quit();
		return TRUE;
	}
	return FALSE;
}

static void
bar_key_bind(cui bar, cui_handler hdr, void *prm)
{
	cui_scbar scb = (cui_scbar)bar;
	cui_bind(scb->dec, CUI_EVT_KEY, hdr, prm);
	cui_bind(scb->dec_pg, CUI_EVT_KEY, hdr, prm);
	cui_bind(scb->thumb, CUI_EVT_KEY, hdr, prm);
	cui_bind(scb->inc_pg, CUI_EVT_KEY, hdr, prm);
	cui_bind(scb->inc, CUI_EVT_KEY, hdr, prm);
}

void
cui_num_init(cui obj, cui parent, int x, int y, int w, int init_v, int min_v, int max_v, int bar_h)
{
	cui_num p = (cui_num)obj;
	char buf[ CUI_ETEXT_BSIZE ];
	cui eview;

	w = w < 6 ? 6 : w;
	cui_base_init(obj, parent, x, y, w, 1);

	if(min_v == max_v){ /* default */
		min_v = INT_MIN / 32;
		max_v = INT_MAX / 32;
	}
	init_v = init_v < min_v ? min_v : init_v;
	init_v = init_v > max_v ? max_v : init_v;
	p->v = init_v;
	p->min_v = min_v;
	p->max_v = max_v;
	p->vp = NULL;
	sprintf(buf, "%d", p->v);	

	if(bar_h == 0) bar_h = w;

	p->fb_dec = cui_fillbtn_new(obj, 0, 0, 1, 1, "<");
	eview = cui_base_new(obj, 1, 0, w-3, 1);
	p->etx = cui_etext_new(eview, -1, 0, w-1, buf);
	p->fb_inc = cui_fillbtn_new(obj, cui_x2(p->etx), 0, 1, 1, ">");
	p->fb_bar = cui_fillbtn_new(obj, cui_x2(p->fb_inc), 0, 1, 1, "|");

	p->bs_bar = cui_base_new(cui_root(obj), 0, 0, 0, 0);
	cui_hide(p->bs_bar);
	p->sc_max = max_v - min_v;
	p->bar = cui_scbar_new(p->bs_bar, 1, 1, 1, bar_h, p->sc_max, 0);
	p->fb_x = cui_fillbtn_new(p->bs_bar, 1, 0, 1, 1, "x");
	cui_wh_fit(p->bs_bar);
	p->bs_bar->w++;

	p->use_dbl = FALSE;

	cui_bind(p->etx, CUI_EVT_BUTTON, cui_num_hdr, obj);
	cui_bind(p->fb_dec, CUI_EVT_BUTTON, cui_num_hdr, obj);
	cui_bind(p->fb_inc, CUI_EVT_BUTTON, cui_num_hdr, obj);
	cui_bind(p->fb_bar, CUI_EVT_BUTTON, cui_num_hdr, obj);

	bar_key_bind(p->bar, bar_key_hdr, obj);
	cui_bind(p->bar, CUI_EVT_BUTTON, cui_num_hdr, obj);
	cui_bind(p->fb_x, CUI_EVT_BUTTON, cui_num_hdr, obj);
}

static void
update(cui obj)
{
	cui_num p = (cui_num)obj;
	char buf[ CUI_ETEXT_BSIZE ];
	char *s = cui_etext_str_get(p->etx);
	
	if(p->use_dbl){
		int i;
		sprintf(buf, "%f", p->dv);
		i = strlen(buf) - 1;
		while(i > 0 && buf[i] == '0') buf[i--] = '\0';
		if(buf[i] == '.') buf[i] = '\0';
	}else if(strncmp(s, "0x", 2) == 0) sprintf(buf, "0x%x", (unsigned int)p->v);
	else sprintf(buf, "%d", p->v);
	cui_etext_str_set(p->etx, buf);
}

static void
num_set(cui obj, int v)
{
	cui_num p = (cui_num)obj;
	if(v < p->min_v) v = p->min_v;
	if(v > p->max_v) v = p->max_v;
	p->v = v;
	if(p->vp) *p->vp = p->v;
}

static void
num_dbl_set(cui obj, double dv)
{
	cui_num p = (cui_num)obj;
	int v;

	if(!p->use_dbl) return;
	if(dv < p->min_dv) dv = p->min_dv;
	if(dv > p->max_dv) dv = p->max_dv;
	p->dv = dv;
	if(p->dvp) *p->dvp = p->dv;

	v = (int)(p->ddiv * (dv - p->min_dv) / (p->max_dv - p->min_dv));
	num_set(obj, v);
}

static double
to_dv(cui obj, int v)
{
	cui_num p = (cui_num)obj;
	return p->min_dv + (p->max_dv - p->min_dv) * v / p->ddiv;
}

int
cui_num_hdr(cui obj, int evt, int val, void *prm)
{
	/* CUI_EVT_BUTTON */

	cui num = (cui)prm;
	cui_num p = (cui_num) num;

	if(obj == p->etx){
		char *s = cui_etext_str_get(p->etx);
		cui_etext_hist_del(p->etx, s);
		if(p->use_dbl) cui_num_dbl_set(num, strtod(s, NULL));
		else cui_num_set(num, strtol(s, NULL, 0));
		cui_etext_hist_add(p->etx);
		cui_handler_call(num, CUI_EVT_BUTTON, p->v);
		return TRUE;
	}else if(obj == p->fb_inc){
		if(p->v >= p->max_v) return TRUE;
		num_set(num, p->v + 1);
		if(p->use_dbl) num_dbl_set(num, to_dv(num, p->v));
		update(num);
		cui_handler_call(num, CUI_EVT_BUTTON, p->v);
		return TRUE;
	}else if(obj == p->fb_dec){
		if(p->v <= p->min_v) return TRUE;
		num_set(num, p->v - 1);
		if(p->use_dbl) num_dbl_set(num, to_dv(num, p->v));
		update(num);
		cui_handler_call(num, CUI_EVT_BUTTON, p->v);
		return TRUE;
	}else if(obj == p->fb_bar){
		cui root = cui_root(obj);
		int x = cui_lx(root, cui_gx(obj)) - 1;
		int y = cui_ly(root, cui_gy(obj)) - p->bs_bar->h / 2;
		cui focus = ((cui_scbar)p->bar)->thumb;

		cui_scbar_val_set(p->bar, p->sc_max - p->v + p->min_v);
		y = y < 0 ? 0 : y;
		cui_xy_set(p->bs_bar, x, y);
		p->bs_bar->flags &= ~CUI_FLG_HIDE;
		cui_main(p->bs_bar, focus);
		cui_hide(p->bs_bar);
		cui_draw(p->bs_bar->parent);
		return TRUE;
	}else if(obj == p->bar){
		num_set(num, p->sc_max - val + p->min_v);
		if(p->use_dbl) num_dbl_set(num, to_dv(num, p->v));
		update(num);
		cui_handler_call(num, CUI_EVT_BUTTON, p->v);
		return TRUE;
	}else if(obj == p->fb_x){
		cui_quit();
		return TRUE;
	}
	return FALSE;
}

int
cui_num_get(cui obj)
{
	cui_num p = (cui_num)obj;
	return p->v;
}

void
cui_num_set(cui obj, int v)
{
	num_set(obj, v);
	update(obj);
}

void
cui_num_vp_set(cui obj, int *vp)
{
	cui_num p = (cui_num)obj;
	p->vp = vp;
	if(p->vp) cui_num_set(obj, *p->vp);
}

void
cui_num_min_max_set(cui obj, int min_v, int max_v)
{
	cui_num p = (cui_num)obj;
	p->min_v = min_v;
	p->max_v = max_v;
	p->sc_max = max_v - min_v;
	cui_num_set(obj, p->v);
}

cui
cui_num_dbl_new(cui parent, int x, int y, int w, double init_dv, double min_dv, double max_dv, double step_dv, int bar_h)
{
	cui obj = cui_alloc(sizeof(struct cui_num));
	cui_num_dbl_init(obj, parent, x, y, w, init_dv, min_dv, max_dv, step_dv, bar_h);
	return obj;
}

void
cui_num_dbl_init(cui obj, cui parent, int x, int y, int w, double init_dv, double min_dv, double max_dv, double step_dv, int bar_h)
{
	cui_num p = (cui_num)obj;
	int ddiv = (step_dv != 0) ? (int)((max_dv - min_dv) / step_dv) : 20;
	int v = (int)(ddiv * (init_dv - min_dv) / (max_dv - min_dv));
	cui_num_init(obj, parent, x, y, w, v, 0, ddiv, bar_h);
	p->use_dbl = TRUE;
	p->dv = init_dv;
	p->min_dv = min_dv;
	p->max_dv = max_dv;
	p->dvp = NULL;
	p->ddiv = ddiv;
	cui_num_dbl_set(obj, init_dv);
}

double
cui_num_dbl_get(cui obj)
{
	cui_num p = (cui_num)obj;
	return p->use_dbl ? p->dv : 0;
}

void
cui_num_dbl_set(cui obj, double dv)
{
	num_dbl_set(obj, dv);
	update(obj);
}

void
cui_num_dbl_vp_set(cui obj, double *dvp)
{
	cui_num p = (cui_num)obj;
	p->dvp = dvp;
	if(p->dvp) cui_num_dbl_set(obj, *p->dvp);
}

void
cui_num_dvp_set(cui obj, double *dvp)
{
	cui_num_dbl_vp_set(obj, dvp);
}

void
cui_num_dbl_min_max_set(cui obj, double min_dv, double max_dv, double step_dv)
{
	cui_num p = (cui_num)obj;
	int ddiv = (step_dv != 0) ? (int)((max_dv - min_dv) / step_dv) : 20;
	cui_num_min_max_set(obj, 0, ddiv);
	p->min_dv = min_dv;
	p->max_dv = max_dv;
	cui_num_dbl_set(obj, p->dv);
}

/* EOF */
