#include "scline.h"
#include "fillbtn.h"
#include "handler.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

cui
cui_scline_new(cui parent, int x, int y, int w)
{
	cui obj = cui_alloc(sizeof(struct cui_scline));
	cui_scline_init(obj, parent, x, y, w);
	return obj;
}

static int
lx(cui obj)
{
	cui_scline p = (cui_scline)obj;
	return p->sheet ? cui_lx(obj, cui_gx(p->sheet)) : 0;
}

static int
lx2(cui obj)
{
	cui_scline p = (cui_scline)obj;
	return p->sheet ? lx(obj) + p->sheet->w : 0;
}

static int
onoff_get(cui obj, int sign)
{
	cui_scline p = (cui_scline)obj;
	return sign < 0 ? 
		!(p->dec->flags & CUI_FLG_HIDE) :
		!(p->inc->flags & CUI_FLG_HIDE);
}

static void
onoff_set(cui obj, int sign, int v)
{
	cui_scline p = (cui_scline)obj;

	if(onoff_get(obj, sign) == v) return;
	if(v){
		if(sign < 0){
			cui_x_set(p->view, p->view->x + 1);
			cui_w_set(p->view, p->view->w - 1);
			if(p->sheet) cui_x_set(p->sheet, p->sheet->x - 1);
		}else{
			cui_w_set(p->view, p->view->w - 1);
		}
		cui_show(sign < 0 ? p->dec : p->inc);
	}else{
		cui_hide(sign < 0 ? p->dec : p->inc);
		if(sign < 0){
			if(p->sheet) cui_x_set(p->sheet, p->sheet->x + 1);
			cui_w_set(p->view, p->view->w + 1);
			cui_x_set(p->view, p->view->x - 1);
		}else{
			cui_w_set(p->view, p->view->w +1);
		}
	}
}

static void
adj(cui obj)
{
	cui_scline p = (cui_scline)obj;

	if(p->sheet == NULL) return;
	if(p->sheet->w <= obj->w){
		onoff_set(obj, -1, FALSE);
		onoff_set(obj, 1, FALSE);
		cui_x_set(p->sheet, 0);
		return;
	}
	if(lx(obj) >= 0){
		onoff_set(obj, -1, FALSE);
		onoff_set(obj, 1, TRUE);
		cui_x_set(p->sheet, 0);
	}else if(lx2(obj) <= obj->w){
		onoff_set(obj, 1, FALSE);
		onoff_set(obj, -1, TRUE);
		cui_x_set(p->sheet, p->view->w - p->sheet->w);
	}else{
		onoff_set(obj, -1, TRUE);
		onoff_set(obj, 1, TRUE);
	}
}

void
cui_scline_init(cui obj, cui parent, int x, int y, int w)
{
	cui_scline p = (cui_scline)obj;

	cui_base_init(obj, parent, x, y, w, 1);

	p->dec = cui_fillbtn_new(obj, 0, 0, 1, 1, "<");
	cui_hide(p->dec);

	p->inc = cui_fillbtn_new(obj, w-1, 0, 1, 1, ">");
	cui_hide(p->inc);

	p->view = cui_base_new(obj, 0, 0, w, 1);
	p->sheet = NULL;

	cui_bind(p->dec, CUI_EVT_BUTTON, cui_scline_hdr, p);
	cui_bind(p->inc, CUI_EVT_BUTTON, cui_scline_hdr, p);
	cui_bind(obj, CUI_EVT_RESIZE, cui_scline_hdr, p);

	p->step_cb = NULL;
	p->step_prm = NULL;
}

int
cui_scline_hdr(cui obj, int evt, int val, void *prm)
{
	cui ev_obj = obj;
	cui_scline p = (cui_scline)prm;
	int step, bak;

	obj = (cui)p;

	switch(evt){
	case CUI_EVT_RESIZE:
		bak = cui_hide_enter(obj);
		if(cui_x2(obj) > obj->parent->w) cui_w_set(obj, obj->parent->w - obj->x);
		cui_x_set(p->inc, obj->w - 1);
		cui_w_set(p->view, obj->w - onoff_get(obj, -1) - onoff_get(obj, 1));
		adj(obj);
		cui_hide_exit(obj, bak);
		break;
	case CUI_EVT_BUTTON:
		if(ev_obj == p->dec){
			if(lx(obj) < 0){
				step = p->step_cb ? (*p->step_cb)(-1, p->step_prm) : 1;
				if(p->sheet) cui_x_set(p->sheet, p->sheet->x + step);
			}
			adj(obj);
		}else if(ev_obj == p->inc){
			if(lx2(obj) > obj->w){
				step = p->step_cb ? (*p->step_cb)(1, p->step_prm) : 1;
				if(p->sheet) cui_x_set(p->sheet, p->sheet->x - step);
			}
			adj(obj);
		}else return FALSE;
		break;		
	default:
		return FALSE;
	}
	return TRUE;
}

cui
cui_scline_view_get(cui obj)
{
	cui_scline p = (cui_scline)obj;
	return p->view;
}

void
cui_scline_sheet_set(cui obj, cui sheet)
{
	cui_scline p = (cui_scline)obj;
	p->sheet = sheet;
}

cui
cui_scline_sheet_get(cui obj)
{
	cui_scline p = (cui_scline)obj;
	return p->sheet;
}

void
cui_scline_step_set(cui obj, int (*step_cb)(int, void *), void *step_prm)
{
	cui_scline p = (cui_scline)obj;
	p->step_cb = step_cb;
	p->step_prm = step_prm;
}

/* EOF */
