#include "tab.h"
#include "handler.h"
#include "key.h"
#include "scline.h"
#include <stdio.h>

cui
cui_tab_btn_new(cui parent, int x, int y, char *s, int v, cui sheet)
{
	cui obj = cui_alloc(sizeof(struct cui_tab_btn));
	cui_tab_btn_init(obj, parent, x, y, s, v, sheet);
	return obj;
}

void
cui_tab_btn_init(cui obj, cui parent, int x, int y, char *s, int v, cui sheet)
{
	cui_radio rd = (cui_radio)obj;
	cui_tab_btn p = (cui_tab_btn)rd;

	cui_radio_init(obj, parent, x, y, s, v);

	cui_hide(obj);

	p->lb1 = rd->lb1;
	p->lb2 = rd->lb2;
	cui_label_str_set(p->lb1, "/ ");
	cui_x_set(p->lb2, cui_x2(p->lb1));
	p->lb3 = cui_label_new(obj, cui_x2(p->lb2), 0, " |");
	p->sheet = sheet;
	cui_wh_fit(obj);
	cui_bind(obj, CUI_EVT_DRAW , cui_tab_btn_hdr, NULL);

	cui_show(obj);
	cui_show_hide(sheet, v);
}

int
cui_tab_btn_hdr(cui obj, int evt, int val, void *prm)
{
	cui_tab_btn p = (cui_tab_btn)obj;
	int attr, v;

	switch(evt){
	case CUI_EVT_DRAW:
		v = cui_tab_btn_val_get(obj);
		attr = v ? CUI_ATTR_NORMAL : CUI_ATTR_ULINE;
		cui_label_attr_set(p->lb1, attr);
		cui_label_attr_set(p->lb3, attr);
		if(val == CUI_DRAW_FOCUS) attr |= CUI_ATTR_REVERSE;
		cui_label_attr_set(p->lb2, attr);
		return TRUE;
	case CUI_EVT_BUTTON:
		cui_show_hide(p->sheet, val);
		return TRUE;
	}
	return FALSE;
}

int
cui_tab_btn_val_get(cui obj)
{
	return cui_radio_val_get(obj);
}

void
cui_tab_btn_val_set(cui obj, int v)
{
	cui_radio_val_set(obj, v);
}

cui
cui_tab_btn_sheet_get(cui obj)
{
	cui_tab_btn p = (cui_tab_btn)obj;
	return p->sheet;
}

void
cui_tab_btn_sheet_set(cui obj, cui sheet)
{
	cui_tab_btn p = (cui_tab_btn)obj;
	p->sheet = sheet;
}

/**/

cui
cui_tab_new(cui parent, int x, int y, int w, char **lst, cui *sheets, int v)
{
	cui obj = cui_alloc(sizeof(struct cui_tab));
	cui_tab_init(obj, parent, x, y, w, lst, sheets, v);
	return obj;
}

static int
tab_hdr(cui obj, int evt, int val, void *prm)
{
	cui_tab p = (cui_tab)obj;
	
	switch(evt){
	case CUI_EVT_RESIZE:
		cui_w_set(p->scl, obj->w);
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

static int
tab_step(int sign, void *step_prm)
{
	cui obj = (cui)step_prm;
	cui_tab p = (cui_tab)obj;
	int n = cui_tab_n_get(obj);
	int i, x, ret = 1;
	cui btn = NULL;
	cui view = cui_scline_view_get(p->scl);
	
	if(sign < 0){
		for(i=n-1; i>=0; i--){
			btn = cui_index_to_child(obj, i);
			x = cui_lx(view, cui_gx(btn));
			if(x < 0) break;
		}
		if(x < 0){
			ret = -x;
			if(i == 0) ret--;
		}
	}else{
		for(i=0; i<n; i++){
			btn = cui_index_to_child(obj, i);
			x = cui_lx(view, cui_gx(btn)) + btn->w;
			if(x > view->w) break;
		}
		if(x > view->w){
			ret = x - view->w;
			if(i == n-1) ret--;
		}
	}
	return ret;
}

void
cui_tab_init(cui obj, cui parent, int x, int y, int w, char **lst, cui *sheets, int v)
{
	cui_tab p = (cui_tab)obj;
	cui btn;
	int i;

	p->scl = cui_scline_new(parent, x, y, w);
	cui_base_init(obj, cui_scline_view_get(p->scl), 0, 0, w, 1);
	cui_scline_sheet_set(p->scl, obj);
	cui_scline_step_set(p->scl, tab_step, p);
	cui_bind(obj, CUI_EVT_RESIZE, tab_hdr, NULL);
	if(lst){
		x = 0;
		for(i=0; lst[i]; i++){
			btn = cui_tab_btn_new(obj, x, 0, lst[i], i == v, sheets[i]);
			x = cui_x2(btn);
			cui_bind(btn, CUI_EVT_BUTTON, cui_tab_btn_hdr, NULL);
		}
	}
	if(w <= 0) cui_wh_fit(obj);
}

void
cui_tab_add(cui obj, char *s, cui sheet, int v)
{
	int first = !obj->children;
	int x = first ? 0 : cui_x2(obj->children);
	cui btn = cui_tab_btn_new(obj, x, 0, s, first, sheet);
	cui_bind(btn, CUI_EVT_BUTTON, cui_tab_btn_hdr, NULL);
	cui_wh_fit(obj);
	cui_draw(obj);
	if(first) cui_show(sheet);
	else if(v) cui_tab_set(obj, sheet);
}

static cui
btn_get(cui obj, cui sheet)
{
	cui c = obj->children;
	for(; c; c=c->next) if(cui_tab_btn_sheet_get(c) == sheet) return c;
	return NULL;
}

void
cui_tab_del(cui obj, cui sheet)
{
	cui clst = obj->children;
	cui c = btn_get(obj, sheet), c2;

	if(!c) return;
	cui_hide(c);
	if(cui_tab_btn_val_get(c)){
		c2 = cui_index_to_child(obj, 0);
		if(c2 == c) c2 = cui_index_to_child(obj, 1);
		if(c2) cui_tab_set(obj, cui_tab_btn_sheet_get(c2));
	}
	for(c2=clst; c2!=c; c2=c2->next) cui_x_set(c2, c2->x - c->w);
	cui_w_set(obj, obj->w - c->w);
	cui_del_free(c);
}

cui
cui_tab_get(cui obj)
{
	cui c = obj->children;
	for(; c; c=c->next) if(cui_tab_btn_val_get(c)) return cui_tab_btn_sheet_get(c);
	return NULL;	
}

void
cui_tab_set(cui obj, cui sheet)
{
	cui c = btn_get(obj, sheet);
	if(!c) return;
	cui_handler_call(c, CUI_EVT_KEY, CUI_KEY_ENTER);
}

cui
cui_tab_scline(cui obj)
{
	cui_tab p = (cui_tab)obj;
	return p->scl;
}

cui
cui_tab_parent(cui obj)
{
	return cui_tab_scline(obj)->parent;
}

int
cui_tab_n_get(cui obj)
{
	return cui_child_num(obj);
}

char *
cui_tab_str_get(cui obj)
{
	cui sheet = cui_tab_get(obj);
	cui btn = btn_get(obj, sheet);
	if(btn == NULL) return "";
	return cui_radio_str_get(btn);
}

/* EOF */
